@@ -1,7 +1,8 @@
 # qemu backend
 
-from UpgradeTestBackend import UpgradeTestBackend
-from DistUpgradeConfigParser import DistUpgradeConfig
+from UpgradeTestBackendSSH import UpgradeTestBackendSSH
+from DistUpgrade.DistUpgradeConfigParser import DistUpgradeConfig
+from DistUpgrade.sourceslist import SourcesList
 
 import ConfigParser
 import subprocess
@@ -16,8 +17,8 @@ import signal
 import crypt
 import tempfile
 import copy
-
-from sourceslist import SourcesList
+import atexit
+import apt_pkg
 
 # images created with http://bazaar.launchpad.net/~mvo/ubuntu-jeos/mvo
 #  ./ubuntu-jeos-builder --vm kvm --kernel-flavor generic --suite feisty --ssh-key `pwd`/ssh-key.pub  --components main,restricted --rootsize 20G
@@ -50,14 +51,9 @@ class NoImageFoundException(Exception):
 class PortInUseException(Exception):
     pass
 
-
-class UpgradeTestBackendQemu(UpgradeTestBackend):
+class UpgradeTestBackendQemu(UpgradeTestBackendSSH):
     " qemu/kvm backend - need qemu >= 0.9.0"
 
-    # FIXME: make this part of the config file
-    #qemu_binary = "qemu"
-    qemu_binary = "kvm"
-    
     qemu_options = [
         "-monitor","stdio",
         "-localtime",
@@ -66,20 +62,31 @@ class UpgradeTestBackendQemu(UpgradeTestBackend):
 #        "-no-kvm",      # crashes sometimes with kvm HW
         ]
 
-    def __init__(self, profile, basedir):
-        UpgradeTestBackend.__init__(self, profile, basedir)
+    def __init__(self, profile):
+        UpgradeTestBackendSSH.__init__(self, profile)
         self.qemu_pid = None
         self.profiledir = os.path.dirname(profile)
+        # get the kvm binary
+        self.qemu_binary = self.config.getWithDefault("KVM","KVM","kvm")
         # setup mount dir/imagefile location
         self.baseimage = self.config.get("KVM","BaseImage")
         if not os.path.exists(self.baseimage):
-            raise NoImageFoundException
+            ret = subprocess.call(["ubuntu-vm-builder","kvm", self.fromDist,
+                                   "--kernel-flavour", "generic",
+                                   "--ssh-key", "%s.pub" % self.ssh_key ,
+                                   "--components", "main,restricted",
+                                   "--rootsize", "80000",
+                                   "--arch", "i386"])
+            # move the disk in place
+            shutil.move("ubuntu-kvm/disk0.qcow2",self.baseimage)
+            if ret != 0:
+                raise NoImageFoundException
         # check if we want virtio here and default to yes
         try:
-            virtio = self.config.getboolean("KVM","Virtio")
+            self.virtio = self.config.getboolean("KVM","Virtio")
         except ConfigParser.NoOptionError,e:
-            virtio = True
-        if virtio:
+            self.virtio = True
+        if self.virtio:
             self.qemu_options.extend(["-net","nic,model=virtio"])
             self.qemu_options.extend(["-net","user"])
         # swapimage
@@ -87,10 +94,12 @@ class UpgradeTestBackendQemu(UpgradeTestBackend):
             self.qemu_options.append("-hdb")
             self.qemu_options.append(self.config.get("KVM","SwapImage"))
         # regular image
-        self.image = os.path.join(self.profiledir, "test-image")
+        profilename = self.config.get("NonInteractive","ProfileName")
+        imagedir = self.config.get("KVM","ImageDir")
+        self.image = os.path.join(imagedir, "test-image.%s" % profilename)
         # make ssh login possible (localhost 54321) available
-        self.ssh_key = os.path.join(self.profiledir,self.config.getWithDefault("NonInteractive","SSHKey","ssh-key"))
         self.ssh_port = self.config.getWithDefault("KVM","SshPort","54321")
+        self.ssh_hostname = "localhost"
         self.qemu_options.append("-redir")
         self.qemu_options.append("tcp:%s::22" % self.ssh_port)
         # vnc port/display
@@ -107,66 +116,8 @@ class UpgradeTestBackendQemu(UpgradeTestBackend):
         if subprocess.call("netstat -t -l -n |grep 0.0.0.0:%s" % self.ssh_port,
                            shell=True) == 0:
             raise PortInUseException, "the port is already in use (another upgrade tester is running?)"
-
-    def _copyToImage(self, fromF, toF, recursive=False):
-        cmd = ["scp",
-               "-P",self.ssh_port,
-               "-q","-q", # shut it up
-               "-i",self.ssh_key,
-               "-o", "StrictHostKeyChecking=no",
-               "-o", "UserKnownHostsFile=%s" % os.path.dirname(self.profile)+"/known_hosts"]
-        if recursive:
-            cmd.append("-r")
-        # we support both single files and lists of files
-        if isinstance(fromF,list):
-            cmd += fromF
-        else:
-            cmd.append(fromF)
-        cmd.append("root@localhost:%s" %  toF)
-        #print cmd
-        ret = subprocess.call(cmd)
-        return ret
-
-    def _copyFromImage(self, fromF, toF):
-        cmd = ["scp",
-               "-P",self.ssh_port,
-               "-q","-q", # shut it up
-               "-i",self.ssh_key,
-               "-o", "StrictHostKeyChecking=no",
-               "-o", "UserKnownHostsFile=%s" % os.path.dirname(self.profile)+"/known_hosts",
-               "root@localhost:%s" %  fromF,
-               toF
-               ]
-        #print cmd
-        ret = subprocess.call(cmd)
-        return ret
-
-
-    def _runInImage(self, command, **kwargs):
-        # ssh -l root -p 54321 localhost -i profile/server/ssh_key
-        #     -o StrictHostKeyChecking=no
-        ret = subprocess.call(["ssh",
-#                               "-tt",
-                               "-l","root",
-                               "-p",self.ssh_port,
-                               "localhost",
-                               "-q","-q", # shut it up
-                               "-i",self.ssh_key,
-                               "-o", "StrictHostKeyChecking=no",
-                               "-o", "UserKnownHostsFile=%s" % os.path.dirname(self.profile)+"/known_hosts",
-                               ]+command, **kwargs)
-        return ret
-
-
-    def installPackages(self, pkgs):
-        " install additional pkgs (list) into the vm before the ugprade "
-        if not pkgs:
-            return True
-        self.start()
-        self._runInImage(["apt-get","update"])
-        ret = self._runInImage(["DEBIAN_FRONTEND=noninteractive","apt-get","install", "--reinstall", "-y"]+pkgs)
-        self.stop()
-        return (ret == 0)
+        # register exit handler to ensure that we quit kvm on exit
+        atexit.register(self.stop)
 
     def genDiff(self):
         """ 
@@ -273,6 +224,14 @@ iface eth0 inet static
             interfaces.flush()
             self._copyToImage(interfaces.name, "/etc/network/interfaces")
         
+        # generate hosts file, the default hosts file contains
+        # "127.0.0.1 ubuntu. ubuntu" for some reason and the missing
+        # domain part after the "." makes e.g. postfix rather unhappy
+        etc_hosts = tempfile.NamedTemporaryFile()
+        etc_hosts.write('127.0.0.1 localhost\n')
+        etc_hosts.write('127.0.0.1 upgrade-test-vm\n')
+        etc_hosts.flush()
+        self._copyToImage(etc_hosts.name, "/etc/hosts")
 
         # generate apt.conf
         proxy = self.config.getWithDefault("NonInteractive","Proxy","")
@@ -294,12 +253,12 @@ iface eth0 inet static
 
         # install some useful stuff
         ret = self._runInImage(["apt-get","update"])
-        assert(ret == 0)
+        assert ret == 0
         # FIXME: instead of this retrying (for network errors with 
         #        proxies) we should have a self._runAptInImage() 
         for i in range(3):
             ret = self._runInImage(["DEBIAN_FRONTEND=noninteractive","apt-get","install", "-y",basepkg]+additional_base_pkgs)
-        assert(ret == 0)
+        assert ret == 0
 
         CMAX = 4000
         pkgs =  self.config.getListFromFile("NonInteractive","AdditionalPkgs")
@@ -319,8 +278,16 @@ iface eth0 inet static
             if os.path.exists(script):
                 self._runInImage(["mkdir","/upgrade-tester"])
                 self._copyToImage(script, "/upgrade-tester")
-                print "running script: %s" % os.path.join("/tmp",script)
-                self._runInImage([os.path.join("/upgrade-tester",script)])
+                self._copyToImage(glob.glob(os.path.dirname(
+                            self.profile)+"/*.cfg"), "/upgrade-tester")
+                script_name = os.path.basename(script)
+                self._runInImage(["chmod","755",
+                                  os.path.join("/upgrade-tester",script_name)])
+                print "running script: %s" % script_name
+                cmd = os.path.join("/upgrade-tester",script_name)
+                ret = self._runInImage(["cd /upgrade-tester; %s" % cmd])
+                print "PostBootstrapScript returned: %s" % ret
+                assert ret == 0, "PostBootstrapScript returned non-zero"
             else:
                 print "WARNING: %s not found" % script
 
@@ -329,11 +296,11 @@ iface eth0 inet static
             print "running apt-get upgrade in from dist (after bootstrap)"
             for i in range(3):
                 ret = self._runInImage(["DEBIAN_FRONTEND=noninteractive","apt-get","-y","dist-upgrade"])
-            assert(ret == 0, "dist-upgrade returned %s" % ret)
+            assert ret == 0, "dist-upgrade returned %s" % ret
 
         print "Cleaning image"
         ret = self._runInImage(["apt-get","clean"])
-        assert(ret == 0, "apt-get clean returned %s" % ret)
+        assert ret == 0, "apt-get clean returned %s" % ret
 
         # done with the bootstrap
         self.stop()
@@ -370,142 +337,117 @@ iface eth0 inet static
         #self.qemu_pid.stdin.write("cont\n")
 
     def start(self):
-        print "Starting %s %s" % (self.qemu_binary, self.qemu_options)
         if self.qemu_pid != None:
             print "already runing"
             return True
-        self.qemu_pid = subprocess.Popen([self.qemu_binary,
-                                          "-hda", self.image,
-                                          ]+self.qemu_options,
-                                         stdin=subprocess.PIPE)
-        
+        # mvo: disabled for now, hardy->lucid does not work well with it
+        #      (random hangs)
+        #if self.virtio:
+        #    drive = ["-drive", "file=%s,if=virtio,boot=on" % self.image]
+        #else:
+        drive = ["-hda", self.image]
+        # build cmd
+        cmd = [self.qemu_binary]+drive+self.qemu_options
+        print "Starting %s" % cmd
+        self.qemu_pid = subprocess.Popen(cmd, stdin=subprocess.PIPE)
         # spin here until ssh has come up and we can login
-        for i in range(900):
+        now = time.time()
+        while True:
             time.sleep(1)
             if self._runInImage(["/bin/true"]) == 0:
                 break
-        else:
-            print "Could not start image after 300s, exiting"
-            return False
+            if (time.time() - now) > 900:
+                print "Could not start image after 900s, exiting"
+                return False
         return True
 
     def stop(self):
         " we stop because we run with -no-reboot"
-        # FIXME: add watchdog here too
-        #        if the qemu process does not stop in sensible time,
-        #        try to umount all FS and then kill it 
+        print "stop"
         if self.qemu_pid:
+            print "stop pid: ", self.qemu_pid
             self._runInImage(["/sbin/reboot"])
             print "waiting for qemu to shutdown"
-            self.qemu_pid.wait()
+            for i in range(600):
+                if self.qemu_pid.poll() is not None:
+                    print "poll() returned"
+                    break
+                time.sleep(1)
+            else:
+                print "Not stopped after 600s, killing "
+                try:
+                    os.kill(int(self.qemu_pid.pid), 15)
+                    time.sleep(10)
+                    os.kill(int(self.qemu_pid.pid), 9)
+                except Exception, e:
+                    print "FAILED to kill %s '%s'" % (self.qemu_pid, e)
             self.qemu_pid = None
             print "qemu stopped"
 
+
     def upgrade(self):
         print "upgrade()"
-        upgrader_args = ""
-        upgrader_env = ""
 
         # clean from any leftover pyc files
-        for f in glob.glob(self.basefilesdir+"/DistUpgrade/*.pyc"):
+        for f in glob.glob("%s/*.pyc" %  self.upgradefilesdir):
             os.unlink(f)
 
         print "Starting for upgrade"
-        self.start()
+        if not self.start():
+            return False
 
-        print "copy upgrader into image"
-        # copy the upgrade into target+/upgrader-tester/
-        files = []
-        self._runInImage(["mkdir","-p","/upgrade-tester","/etc/update-manager/release-upgrades.d"])
-        for f in glob.glob("%s/DistUpgrade/*" % self.basefilesdir):
-            if not os.path.isdir(f):
-                files.append(f)
-            elif os.path.islink(f):
-                print "Copying link '%s' to image " % f
-                self._copyToImage(f, "/upgrade-tester", recursive=True)
-        self._copyToImage(files, "/upgrade-tester")
-        # copy the patches
-        d="%s/DistUpgrade/patches" % self.basefilesdir
-        print "Copying '%s' to image" % d
-        self._copyToImage(d, "/upgrade-tester/", recursive=True)
         # copy the profile
         if os.path.exists(self.profile):
             print "Copying '%s' to image overrides" % self.profile
+            self._runInImage(["mkdir","-p","/etc/update-manager/release-upgrades.d"])
             self._copyToImage(self.profile, "/etc/update-manager/release-upgrades.d/")
-        # and any other cfg files
-        for f in glob.glob(os.path.dirname(self.profile)+"/*.cfg"):
-            if (os.path.isfile(f) and
-                not os.path.basename(f).startswith("DistUpgrade.cfg")):
-                print "Copying '%s' to image " % f
-                self._copyToImage(f, "/upgrade-tester")
-        # and prereq lists
-        prereq = self.config.getWithDefault("PreRequists","SourcesList",None)
-        if prereq is not None:
-            prereq = os.path.join(os.path.dirname(self.profile),prereq)
-            print "Copying '%s' to image" % prereq
-            self._copyToImage(prereq, "/upgrade-tester")
 
-        # this is to support direct copying of backport udebs into the 
-        # qemu image - useful for testing backports without having to
-        # push them into the archive
-        backports = self.config.getlist("NonInteractive", "PreRequistsFiles")
-        if backports:
-            self._runInImage(["mkdir -p /upgrade-tester/backports"])
-            for f in backports:
-                print "Copying %s" % os.path.basename(f)
-                self._copyToImage(f, "/upgrade-tester/backports/")
-                self._runInImage(["(cd /upgrade-tester/backports ; dpkg-deb -x %s . )" % os.path.basename(f)])
-            upgrader_args = " --have-prerequists"
-            upgrader_env = "LD_LIBRARY_PATH=/upgrade-tester/backports/usr/lib PATH=/upgrade-tester/backports/usr/bin:$PATH PYTHONPATH=/upgrade-tester/backports//usr/lib/python$(python -c 'import sys; print \"%s.%s\" % (sys.version_info[0], sys.version_info[1])')/site-packages/ "
-
-        # copy test repo sources.list (if available)
+        # copy test repo sources.list (if needed) 
         test_repo = self.config.getWithDefault("NonInteractive","AddRepo","")
         if test_repo:
             test_repo = os.path.join(os.path.dirname(self.profile), test_repo)
             self._copyToImage(test_repo, "/etc/apt/sources.list.d")
+            sourcelist = self.getSourcesListFile()
+            apt_pkg.Config.Set("Dir::Etc", os.path.dirname(sourcelist.name))
+            apt_pkg.Config.Set("Dir::Etc::sourcelist", 
+                               os.path.basename(sourcelist.name))
             sources = SourcesList(matcherPath=".")
             sources.load(test_repo)
             # add the uri to the list of valid mirros in the image
+            self._runInImage(["mkdir","-p","/upgrade-tester"])
+            self._runInImage(["echo -e '[Sources]\nValidMirrors=/upgrade-tester/new_mirrors.cfg' > /etc/update-manager/release-upgrades.d/new_mirrors.cfg"])
             for entry in sources.list:
                 if (not (entry.invalid or entry.disabled) and
                     entry.type == "deb"):
                     print "adding %s to mirrors" % entry.uri
-                    self._runInImage(["echo '%s' >> /upgrade-tester/mirrors.cfg" % entry.uri])
+                    self._runInImage(["echo '%s' >> /upgrade-tester/new_mirrors.cfg" % entry.uri])
+                    
 
-        # start the upgrader
-        print "running the upgrader now"
-        ret = self._runInImage(["(cd /upgrade-tester/ ; "
-                                "%s./dist-upgrade.py %s)" % (upgrader_env, upgrader_args)])
+        # check if we have a bzr checkout dir to run against or
+        # if we should just run the normal upgrader
+        cmd_prefix=[]
+        if not self.config.getWithDefault("NonInteractive","ForceOverwrite", False):
+            print "Disabling ForceOverwrite"
+            cmd_prefix = ["export RELEASE_UPGRADE_NO_FORCE_OVERWRITE=1;"]
+        if os.path.exists(self.upgradefilesdir):
+            self._copyUpgraderFilesFromBzrCheckout()
+            ret = self._runBzrCheckoutUpgrade(cmd_prefix)
+        else:
+            ret = self._runInImage(cmd_prefix+["do-release-upgrade","-d",
+                                    "-f","DistUpgradeViewNonInteractive"])
         print "dist-upgrade.py returned: %i" % ret
 
         # copy the result
         print "coyping the result"
         self._copyFromImage("/var/log/dist-upgrade/*",self.resultdir)
 
+        # give the ssh output extra time
+        time.sleep(10)
+
         # stop the machine
         print "Shuting down the VM"
         self.stop()
-
         return (ret == 0)
-
-    def test(self):
-        # FIXME: add some tests here to see if the upgrade worked
-        # this should include:
-        # - new kernel is runing (run uname -r in target)
-        # - did it sucessfully rebootet
-        # - is X runing
-        # - generate diff of upgrade vs fresh install
-        # ...
-        #self.genDiff()
-        self.start()
-        self._copyFromImage("/var/crash/*.crash", self.resultdir)
-        crashfiles = glob.glob(self.resultdir+"/*.crash")
-        self.stop()
-        if len(crashfiles) > 0:
-            print "WARNING: crash files detected on the upgrade"
-            print crashfiles
-            return False
-        return True
         
 
 if __name__ == "__main__":
