@@ -83,7 +83,7 @@ class DumbTerminal(QTextEdit):
             os.environ["TERM"] = "dumb"
         return self.child_pid
 
-    def updateInterface(self):
+    def update_interface(self):
         (rlist, wlist, xlist) = select.select([self.installProgress.master_fd],[],[], 0)
         if len(rlist) > 0:
             line = os.read(self.installProgress.master_fd, 255)
@@ -154,7 +154,7 @@ class KDECdromProgressAdapter(apt.progress.CdromProgress):
     def changeCdrom(self):
         return False
 
-class KDEOpProgress(apt.progress.OpProgress):
+class KDEOpProgress(apt.progress.base.OpProgress):
   """ methods on the progress bar """
   def __init__(self, progressbar, progressbar_label):
       self.progressbar = progressbar
@@ -186,7 +186,7 @@ class KDEFetchProgressAdapter(FetchProgress):
         self.progress = parent.window_main.progressbar_cache
         self.parent = parent
 
-    def mediaChange(self, medium, drive):
+    def media_change(self, medium, drive):
       msg = _("Please insert '%s' into the drive '%s'") % (medium,drive)
       change = QMessageBox.question(self.parent.window_main, _("Media Change"), msg, QMessageBox.Ok, QMessageBox.Cancel)
       if change == QMessageBox.Ok:
@@ -194,6 +194,7 @@ class KDEFetchProgressAdapter(FetchProgress):
       return False
 
     def start(self):
+        FetchProgress.start(self)
         #self.progress.show()
         self.progress.setValue(0)
         self.status.show()
@@ -202,21 +203,21 @@ class KDEFetchProgressAdapter(FetchProgress):
         self.parent.window_main.progress_text.setText("  ")
         self.status.setText(_("Fetching is complete"))
 
-    def pulse(self):
+    def pulse(self, owner):
         """ we don't have a mainloop in this application, we just call processEvents here and elsewhere"""
         # FIXME: move the status_str and progress_str into python-apt
         # (python-apt need i18n first for this)
-        FetchProgress.pulse(self)
+        FetchProgress.pulse(self, owner)
         self.progress.setValue(self.percent)
-        currentItem = self.currentItems + 1
-        if currentItem > self.totalItems:
-            currentItem = self.totalItems
+        current_item = self.current_items + 1
+        if current_item > self.total_items:
+            current_item = self.total_items
 
-        if self.currentCPS > 0:
-            self.status.setText(_("Fetching file %li of %li at %sB/s") % (currentItem, self.totalItems, apt_pkg.SizeToStr(self.currentCPS)))
+        if self.current_cps > 0:
+            self.status.setText(_("Fetching file %li of %li at %sB/s") % (current_item, self.total_items, apt_pkg.size_to_str(self.current_cps)))
             self.parent.window_main.progress_text.setText("<i>" + _("About %s remaining") % unicode(FuzzyTimeToStr(self.eta), 'utf-8') + "</i>")
         else:
-            self.status.setText(_("Fetching file %li of %li") % (currentItem, self.totalItems))
+            self.status.setText(_("Fetching file %li of %li") % (current_item, self.total_items))
             self.parent.window_main.progress_text.setText("  ")
 
         QApplication.processEvents()
@@ -242,10 +243,10 @@ class KDEInstallProgressAdapter(InstallProgress):
             logging.error("Can not open terminal log: '%s'" % e)
             self._terminal_log = sys.stdout
         # some options for dpkg to make it die less easily
-        apt_pkg.Config.Set("DPkg::StopOnError","False")
+        apt_pkg.config.set("DPkg::StopOnError","False")
 
-    def startUpdate(self):
-        InstallProgress.startUpdate(self)
+    def start_update(self):
+        InstallProgress.start_update(self)
         self.finished = False
         # FIXME: add support for the timeout
         # of the terminal (to display something useful then)
@@ -266,9 +267,9 @@ class KDEInstallProgressAdapter(InstallProgress):
         if gettext.dgettext('dpkg', "dependency problems - leaving unconfigured") in errormsg:
           return False
         summary = _("Could not install '%s'") % pkg
-        msg = _("The upgrade will continue but the '%s' package may be "
-                "in a not working state. Please consider submitting a "
-                "bugreport about it.") % pkg
+        msg = _("The upgrade will continue but the '%s' package may not "
+                "be in a working state. Please consider submitting a "
+                "bug report about it.") % pkg
         msg = "<big><b>%s</b></big><br />%s" % (summary, msg)
 
         dialogue = QDialog(self.parent.window_main)
@@ -299,6 +300,11 @@ class KDEInstallProgressAdapter(InstallProgress):
         #FIXME, below to be tested
         #self.confDialogue.resize(self.confDialogue.minimumSizeHint())
         self.confDialogue.connect(self.confDialogue.show_difference_button, SIGNAL("clicked()"), self.showConffile)
+        self.confDialogue.show_difference_button.setEnabled(False)
+
+        # workaround silly dpkg 
+        if not os.path.exists(current):
+          current = current+".dpkg-dist"
 
         # now get the diff
         if os.path.exists("/usr/bin/diff"):
@@ -335,7 +341,7 @@ class KDEInstallProgressAdapter(InstallProgress):
         logging.debug(" fork pid is: %s" % self.child_pid)
         return self.child_pid
 
-    def statusChange(self, pkg, percent, status):
+    def status_change(self, pkg, percent, status):
         """update progress bar and label"""
         # start the timer when the first package changes its status
         if self.start_time == 0.0:
@@ -358,10 +364,10 @@ class KDEInstallProgressAdapter(InstallProgress):
           else:
             self.progress_text.setText(" ")
 
-    def finishUpdate(self):
+    def finish_update(self):
         self.label_status.setText("")
 
-    def updateInterface(self):
+    def update_interface(self):
         """
         no mainloop in this application, just call processEvents lots here
         it's also important to sleep for a minimum amount of time
@@ -383,9 +389,9 @@ class KDEInstallProgressAdapter(InstallProgress):
 
         # now update the GUI
         try:
-          InstallProgress.updateInterface(self)
+          InstallProgress.update_interface(self)
         except ValueError, e:
-          logging.error("got ValueError from InstallProgress.updateInterface. Line was '%s' (%s)" % (self.read, e))
+          logging.error("got ValueError from InstallProgress.update_interface. Line was '%s' (%s)" % (self.read, e))
           # reset self.read so that it can continue reading and does not loop
           self.read = ""
         # check about terminal activity
@@ -402,13 +408,14 @@ class KDEInstallProgressAdapter(InstallProgress):
         QApplication.processEvents()
         time.sleep(0.02)
 
-    def waitChild(self):
+    def wait_child(self):
         while True:
-            self.updateInterface()
+            self.update_interface()
             (pid, res) = os.waitpid(self.child_pid,os.WNOHANG)
             if pid == self.child_pid:
                 break
-        return os.WEXITSTATUS(res)
+        # we need the full status here (not just WEXITSTATUS)
+        return res
 
 # inherit from the class created in window_main.ui
 # to add the handler for closing the window
@@ -432,6 +439,9 @@ class UpgraderMainWindow(QWidget):
 class DistUpgradeViewKDE(DistUpgradeView):
     """KDE frontend of the distUpgrade tool"""
     def __init__(self, datadir=None, logdir=None):
+        # silence the PyQt4 logger
+        logger = logging.getLogger("PyQt4")
+        logger.setLevel(logging.INFO)
         if not datadir:
           localedir=os.path.join(os.getcwd(),"mo")
         else:
@@ -450,7 +460,10 @@ class DistUpgradeViewKDE(DistUpgradeView):
         #about.addAuthor("Michael Vogt", None,"michael.vogt@ubuntu.com")
         #KCmdLineArgs.init(["./dist-upgrade.py"],about)
 
-        #self.app = KApplication()
+        # we test for DISPLAY here, QApplication does not throw a 
+        # exception when run without DISPLAY but dies instead
+        if not "DISPLAY" in os.environ:
+            raise Exception, "No DISPLAY in os.environ found"
         self.app = QApplication(["update-manager"])
 
         if os.path.exists("/usr/share/icons/oxygen/48x48/apps/system-software-update.png"):
@@ -478,7 +491,7 @@ class DistUpgradeViewKDE(DistUpgradeView):
         self.app.connect(self.window_main.showTerminalButton, SIGNAL("clicked()"), self.showTerminal)
 
         #kdesu requires us to copy the xauthority file before it removes it when Adept is killed
-        copyXauth = tempfile.mktemp("", "adept")
+        fd, copyXauth = tempfile.mkstemp("", "adept")
         if 'XAUTHORITY' in os.environ and os.environ['XAUTHORITY'] != copyXauth:
             shutil.copy(os.environ['XAUTHORITY'], copyXauth)
             os.environ["XAUTHORITY"] = copyXauth
@@ -607,7 +620,7 @@ class DistUpgradeViewKDE(DistUpgradeView):
     def getCdromProgress(self):
         return self._cdromProgress
 
-    def updateStatus(self, msg):
+    def update_status(self, msg):
         self.window_main.label_status.setText(utf8(msg))
 
     def hideStep(self, step):
@@ -708,12 +721,13 @@ class DistUpgradeViewKDE(DistUpgradeView):
 
         return False
 
-    def confirmChanges(self, summary, changes, downloadSize, 
+    def confirmChanges(self, summary, changes, demotions, downloadSize, 
                        actions=None, removal_bold=True):
         """show the changes dialogue"""
         # FIXME: add a whitelist here for packages that we expect to be
         # removed (how to calc this automatically?)
-        DistUpgradeView.confirmChanges(self, summary, changes, downloadSize)
+        DistUpgradeView.confirmChanges(self, summary, changes, demotions, 
+                                       downloadSize)
         msg = unicode(self.confirmChangesMessage, 'UTF-8')
         self.changesDialogue = QDialog(self.window_main)
         loadUi("dialog_changes.ui", self.changesDialogue)
@@ -746,8 +760,12 @@ class DistUpgradeViewKDE(DistUpgradeView):
         self.changesDialogue.treeview_details.clear()
         self.changesDialogue.treeview_details.setHeaderLabels(["Packages"])
         self.changesDialogue.treeview_details.header().hide()
+        for demoted in self.demotions:
+            self.changesDialogue.treeview_details.insertTopLevelItem(0, QTreeWidgetItem(self.changesDialogue.treeview_details, [_("No longer supported %s") % demoted]) )
         for rm in self.toRemove:
             self.changesDialogue.treeview_details.insertTopLevelItem(0, QTreeWidgetItem(self.changesDialogue.treeview_details, [_("Remove %s") % rm]) )
+        for rm in self.toRemoveAuto:
+            self.changesDialogue.treeview_details.insertTopLevelItem(0, QTreeWidgetItem(self.changesDialogue.treeview_details, [_("Remove (was auto installed) %s") % rm]) )
         for inst in self.toInstall:
             self.changesDialogue.treeview_details.insertTopLevelItem(0, QTreeWidgetItem(self.changesDialogue.treeview_details, [_("Install %s") % inst]) )
         for up in self.toUpgrade:
@@ -788,6 +806,12 @@ class DistUpgradeViewKDE(DistUpgradeView):
     def processEvents(self):
         QApplication.processEvents()
 
+    def pulseProgress(self, finished=False):
+        # FIXME: currently we do nothing here because this is
+        # run in a different python thread and QT explodes if the UI is
+        # touched from a non QThread
+        pass
+
     def on_window_main_delete_event(self):
         #FIXME make this user friendly
         text = _("""<b><big>Cancel the running upgrade?</big></b>
@@ -800,15 +824,17 @@ The system could be in an unusable state if you cancel the upgrade. You are stro
         return False
 
 if __name__ == "__main__":
+  
+  view = DistUpgradeViewKDE()
+  view.askYesNoQuestion("input box test","bar bar bar bar bar bar bar bar bar bar bar bar bar bar bar bar bar bar bar bar bar bar bar bar bar bar bar bar bar bar bar bar bar bar bar bar bar bar bar bar bar bar bar bar bar bar bar bar bar bar bar bar bar bar bar bar bar bar bar bar bar ")
 
   if sys.argv[1] == "--test-term":
-      view = DistUpgradeViewKDE4()
       pid = view.terminal_text.fork()
       if pid == 0:
           subprocess.call(["bash"])
           sys.exit()
       while True:
-          view.terminal_text.updateInterface()
+          view.terminal_text.update_interface()
           QApplication.processEvents()
           time.sleep(0.01)
 
@@ -821,7 +847,6 @@ if __name__ == "__main__":
       while True:
           QApplication.processEvents()
 
-  view = DistUpgradeViewKDE4()
   cache = apt.Cache()
   for pkg in sys.argv[1:]:
     if cache[pkg].isInstalled and not cache[pkg].isUpgradable: 
